<?php

namespace Teknisa\FIN\Services;

use Doctrine\ORM\EntityManager;
use Geral\Entities\Tipoctreceber;
use Geral\Entities\Tipoctpagar;
use Teknisa\FIN\Factories\ContabilizacaoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\Masks\PictureProduto;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response;
use Teknisa\Libs\Util\NewCode;

class ContabilizacaoService {

    protected $contabilizacaoFactory;
    protected $entityManager;
    protected $maskService;
    protected $nrorg;
    protected $newCode;

    // Variáveis de armazenamento dos dados temporários dos erros.
    protected $errosContabilizacao = array();
    protected $partidasContabilizacao = array();
    protected $rateiosContabilizacao = array();

    // Mensagens dos erros por código
    protected $errosPadroes = array(
        '001' => 'Total de crédito difere do total de débito.',
        '002' => 'Data fora do período contábil.',
        '003' => 'Partida %s - Conta contábil não cadastrada.',
        '004' => 'Partida %s - Histórico padrão não cadastrado.',
        '005' => 'Partida %s - Código comercial não cadastrado.',
        '006' => 'Partida %s - Valor da partida zerado.',
        '007' => 'Partida %s - Valor de rateio zerado.',
        '008' => 'Partida %s - Filial de rateio não cadastrada.',
        '009' => 'Partida %s - Centro de custo não cadastrado para filial informada.',
        '010' => 'Partida %s - Soma dos rateios difere do valor da partida.',
        '011' => 'Partida %s - Conta contábil sem código comercial informado.',
        '012' => 'Partida %s - Conta Contábil Gerencial sem rateio informado.',
        '013' => 'Partida %s - Centro de custo deve ser informado.',
        '014' => 'Produto %s sem conta contábil.',
        '015' => 'Lançamento fora do período contábil para o financeiro. (BCO, PAG e REC)',
        '016' => 'Lançamento fora do período contábil de documentos fiscais de entrada. (DFE)',
        '017' => 'Lançamento fora do período contábil para documentos fiscais de saída. (DFS)'
    );

    public function __construct(EntityManager $entityManager, Environment $environment, Masks $maskService, ContabilizacaoFactory $contabilizacaoFactory, NewCode $newCode) {
        $this->contabilizacaoFactory = $contabilizacaoFactory;
        $this->entityManager = $entityManager;
        $this->maskService = $maskService;
        $this->nrorg = $environment->getNrOrgTrab();
        $this->newCode                   = $newCode;
    }


    /********************************************************************
     *                                                                  *
     *                     Funções de Contabilização                    *
     *                                                                  *
     ********************************************************************/

    /*
     * Função para a contabilização de recebimento.
     */
    public function contabilizaRecebimento(
        $cdfilial, $cdempresa, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $idstatrec, $cdtpctreceb,
        $idutlanccrec = 'S', $dtajusterec = null, $cdajuctrec = null, $idtipo = 'D'
    ) {
        // Busca o tipo de conta a receber do lançamento.
        $tipoctreceber = Tipoctreceber::findOneByNrorgAndCdtpctrecebAndIdutlanccrec($this->nrorg, $cdtpctreceb, $idutlanccrec);
    
        // Verifica parâmetros "Utilização Contábil" e "Status Conta Receber".
        // Esses parâmetros determinam se o lançamento irá gerar contabilização.
        $nrlancctb = null;
        if ($tipoctreceber->getIdctblctrec() == 'S' && $idstatrec == 'A') {
            // Contabiliza conta a receber.
            $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbRecLanc(
                $cdfilial, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $dtajusterec, $cdajuctrec, $idtipo
            );
        };
        
        // Verifica se a função de contabilização foi chamada.
        if ($nrlancctb !== null) {
            // Caso haja algum erro de contabilização, a função irá lançar uma exceção.
            $this->checkErrosContabilizacao($cdempresa, $nrlancctb);
        }
    }

    /*
     * Função para a contabilização de baixa a receber.
     */
    public function contabilizaBaixaReceber($cdempresa, $cdfilial, $cdfiliproc, $nrbaixarec) {
        // Busca Empresa
        $empresa = $this->contabilizacaoFactory->buscaEmpresa($cdempresa);
        
        // Busca Baixa Recebida
        $baixarecebida = $this->contabilizacaoFactory->buscaBaixarecebida($cdempresa, $cdfilial, $nrbaixarec);
        
        // Busca Tipo de Baixa Recebida
        $tipobaixarec = $this->contabilizacaoFactory->buscaTipobaixarec($baixarecebida->getCdtpbaixarec());
        
        // Valida e Contabiliza Baixa Recebida
        if($empresa->getIdctbbaixaempr() == 'S' && $tipobaixarec->getIdctblbxrec() == 'S')
            $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbRecBaixa($cdempresa, $cdfilial, $cdfiliproc, $nrbaixarec);
        else $nrlancctb = null;

        if(substr($nrlancctb,0,1) == 'N'){
            $nrlancctb =  substr($nrlancctb,1);
        }
        
        // Caso haja algum erro de contabilização, a função irá lançar uma exceção.
        $this->checkErrosContabilizacao($cdempresa, $nrlancctb);
    }

    /*
     * Função para a contabilização de lançamento de controle bancário.
     */
    public function contabilizaLancamentoControleBancario($cdempresa, $cdreducont, $nrmovi, $idtipo, $idctblbanc, $cdcentcust = null, $nrseqlancdesp = 0) {
        // Verifica se o lançamento deve ser contabilizado.
        $nrlancctb = null;
        if ($idctblbanc == 'S') {
            // Contabiliza lançamento de controle bancário.
            $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbBcoLanc($cdreducont, $nrmovi, $idtipo, $cdcentcust, $nrseqlancdesp);
        }
        
        // Verifica se a função de contabilização foi chamada.
        if ($nrlancctb !== null) {
            // Caso haja algum erro de contabilização, a função irá lançar uma exceção.
            $this->checkErrosContabilizacao($cdempresa, $nrlancctb);
        }
    }

    /*
     * Função para a contabilização de notas fiscais.
     */
    public function contabilizaNotasFiscais($cdempresa, $identrsaidop, $cdfilial, $nrlanctonf) {
        // Verifica se a nota informada é de entrada.
        if ($identrsaidop == 'E') {
            // Contabiliza nota fiscal de entrada.
            $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbDfeContab($cdfilial, $nrlanctonf);
        }
        else {
            // Contabiliza nota fiscal de saída.
            $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbDfsContab($cdfilial, $nrlanctonf);
        }

        // Caso haja algum erro de contabilização, a função irá lançar uma exceção.
        $this->checkErrosContabilizacao($cdempresa, $nrlancctb);
    }

    /*
     * Função para a contabilização de pagamento.
     */
    public function contabilizaPagamento(
        $cdfilial, $cdempresa, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $idstatpag, $cdtpctpagar,
        $idutlanccpag = 'S', $dtajustepag = null, $cdajuctpag = null, $idtipo = 'D'
    ) {
        // Busca o tipo de conta a pagar do lançamento.
        $tipoctpagar = Tipoctpagar::findOneByNrorgAndCdtpctpagarAndIdutlanccpag($this->nrorg, $cdtpctpagar, $idutlanccpag);
    
        // Verifica parâmetros "Utilização Contábil" e "Status Conta Pagar".
        // Esses parâmetros determinam se o lançamento irá gerar contabilização.
        $nrlancctb = null;
        if ($tipoctpagar->getIdctblctpag() == 'S' && $idstatpag == 'A') {
            // Contabiliza conta a pagar.
            $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbPagLanc(
                $cdfilial, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $dtajustepag, $cdajuctpag, $idtipo
            );
        };

        // Verifica se a função de contabilização foi chamada.
        if ($nrlancctb !== null) {
            // Caso haja algum erro de contabilização, a função irá lançar uma exceção.
            $this->checkErrosContabilizacao($cdempresa, $nrlancctb);
        }
    }

    /*
     * Função para a contabilização de baixa a pagar.
     */
    public function contabilizaBaixaPagar($cdempresa, $cdfilial, $nrbaixapag) {
        // Busca Empresa
        $empresa = $this->contabilizacaoFactory->buscaEmpresa($cdempresa);
        
        // Busca Baixa Paga
        $baixapaga = $this->contabilizacaoFactory->buscaBaixapaga($cdempresa, $cdfilial, $nrbaixapag);
        
        // Busca Tipo de Baixa Paga
        $tipobaixapag = $this->contabilizacaoFactory->buscaTipobaixapag($baixapaga->getCdtpbaixapag());
        
        // Valida e Contabiliza Baixa Paga
        if($empresa->getIdctbbaixaempr() == 'S' && $tipobaixapag->getIdctblbxpag() == 'S')
            $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbBaixaPag($cdempresa, $cdfilial, $nrbaixapag);
        else $nrlancctb = null;
        
        // Caso haja algum erro de contabilização, a função irá lançar uma exceção.
        $this->checkErrosContabilizacao($cdempresa, $nrlancctb);
    }

    /*
     * Função para a contabilização de transferência bancária.
     */
    public function contabilizaTransferenciaBancaria($cdempresa, $nrtranbanc, $dttranbanc, $cdfiliproc) {
        // Busca o os parâmetros bancários.
        $parametrosBancarios = $this->contabilizacaoFactory->getParametrosBancarios($this->nrorg);

        // Verifica parâmetro que determina contabilização da transferência.
        $nrlancctb = null;
        if(isset($parametrosBancarios["IDCTBTRANS"])){
            if ($parametrosBancarios["IDCTBTRANS"] == 'S') {
                // Contabiliza transferência.
                $nrlancctb = $this->contabilizacaoFactory->callProcedureCtbBcoTransf($cdempresa, $nrtranbanc, $dttranbanc, $cdfiliproc);
            };
        }else throw new \Exception("Operação bloqueada. Parâmetros de Controle Bancário não encontrados.");

        // Verifica se a função de contabilização foi chamada.
        if ($nrlancctb !== null) {
            // Caso haja algum erro de contabilização, a função irá lançar uma exceção.
            $this->checkErrosContabilizacao($cdempresa, $nrlancctb);
        }
    }



    /********************************************************************
     *                                                                  *
     *                Funções de Manipulação da Requisição              *
     *                                                                  *
     ********************************************************************/

    /*
     * Função para verificação de erros, para que, caso haja algum erro de
     * contabilização,seja lançada umaa exceção que deve ser tratada
     * na classe em que a contabilização foi chamada.
     * Armazena os dados referentes aos erros para o envio desses na resposta
     * da requisição, como dataSets.
     */
    private function checkErrosContabilizacao($cdempresa, $nrlancctb) {
        // Salva os erros no array, permanecendo vazio se não houver erros.
        $errosContabilizacao = $this->getErros($cdempresa, $nrlancctb);
        
        // Se houver algum erro no array, lança a exceção e salva os dados relacionados.
        if (!empty($errosContabilizacao)) {
            // Salva os dados temporários em variáveis.
            // Se já houver dados de outro lançamento, adiciona aos dados anteriores.
            $this->errosContabilizacao = array_merge($this->errosContabilizacao, $errosContabilizacao);
            $this->partidasContabilizacao = array_merge($this->partidasContabilizacao, $this->getPartidas($cdempresa, $nrlancctb));
            $this->rateiosContabilizacao = array_merge($this->rateiosContabilizacao, $this->getRateios($cdempresa, $nrlancctb));

            // Remove os dados temporários das tabelas no banco de dados.
            //$this->deleteDadosTemporarios($cdempresa, $nrlancctb);

            throw new \Exception("Erro de contabilização.");
        }
    }

    public function getContabilizacaoLancamentos($lancamentos) {
        // Inicializa os arrays para armazenamento de dados.
        $partidas = array();
        $rateios = array();

        // Inicializa array de parâmetros.
        $params = array(
            "NRORG" => $this->nrorg
        );

        // Busca os dados de cada lançamento.
        foreach ($lancamentos as $lancamento) {
            // Verifica se os dados foram inseridos corretamente.
            if (!(empty($lancamento["NRLANCCONT"]) || empty($lancamento["CDEMPRESA"]))) {
                $params["NRLANCCONT"] = $lancamento["NRLANCCONT"];
                $params["CDEMPRESA"] = $lancamento["CDEMPRESA"];

                // Adiciona os dados de partidas e rateios nos respectivos arrays de armazenamento.
                $partidas = array_merge($partidas, $this->contabilizacaoFactory->getPartidasLancamento($params));
                $rateios = array_merge($rateios, $this->contabilizacaoFactory->getRateiosLancamento($params));
            }
        }

        return array("partidas" => $partidas, "rateios" => $rateios);
    }

    /*
     * Função para retornar os erros de contabilização de um lançamento
     */
    public function getErrosContabilizacao() {
        return $this->errosContabilizacao;
    }

    /*
     * Função para retornar as partidas de um lançamento com erros
     */
    public function getPartidasContabilizacao() {
        return $this->partidasContabilizacao;
    }

    /*
     * Função para retornar os rateios de um lançamento com erros
     */
    public function getRateiosContabilizacao() {
        return $this->rateiosContabilizacao;
    }

    /*
     * Função que irá setar os dataSets no response da requisição do usuário.
     * Isso é necessário para que, havendo erros de contabilização, os dados
     * temporários possam ser excluídos, mas ainda sejam exibidos no frontend,
     * sendo este o meio de acesso a esses dados após a exclusão.
     * Caso não haja qualquer erro de contabilização e, ainda assim, essa função
     * seja chamada, os dataSets adicionados não conterão dados.
     */
    public function saveContabilizacaoData(Response $response) {
        $response->addDataSet(new DataSet('errosContabilizacao', $this->errosContabilizacao));
        $response->addDataSet(new DataSet('partidasContabilizacao', $this->partidasContabilizacao));
        $response->addDataSet(new DataSet('rateiosContabilizacao', $this->rateiosContabilizacao));
    }


    /********************************************************************
     *                                                                  *
     *                 Funções de Manipulação dos Dados                 *
     *                                                                  *
     ********************************************************************/

    /*
     * Função para retornar um array com as mensagens e códigos dos erros que
     * ocorreram no lançamento.
     * Se não houver erros no lançamento, retorna um array vazio.
     */
    private function getErros($cdempresa, $nrlancctb) {
        $erroctbtemp = $this->contabilizacaoFactory->getErrosTemporarios(array(
            "NRORG" => $this->nrorg,
            "CDEMPRESA" => $cdempresa,
            "NRLANCCTBTEMP" => $nrlancctb
        ));
        // Chamada da função que irá modificar as mensagens que possuem '%s'.
        $erroctbtemp = $this->getParameterizedErrorMessages($erroctbtemp);
        return $erroctbtemp;
    }

    /*
     * Função para incluir as descrições das mensagens já parametrizadas
     * de acordo com seus códigos de erro.
     */
    private function getParameterizedErrorMessages($errorsArray) {
        // Armazena a máscara para formatar os códigos do produtos com erros, se houver.
        $productMask = $this->getProductMask();

        foreach ($errorsArray as &$error) {

            // A mensagem é buscada pelo código no array de erros padrões.
            $message = $this->errosPadroes[explode('|', $error["CDERROTEMP"])[0]];
            // Se a mensagem possuir o identificador '%s', substitui-o com seu parâmetro.
             if (strpos($message, '%s')) {
                 // Verifica se o parâmetro é de produto.
                 if ($error["CDERROTEMP"] === '014') {
                    // Neste caso, o código do produto é armazenado com o código
                    // do erro, separado por '|'.
                    $errDesc = explode('|', $error["CDERROTEMP"]);
                    $error["CDERROTEMP"] = $errDesc[0];
                    $replaceParameter = $errDesc[1];
                    // Utiliza o utilitário Masks para formatar o código do produto.
                    $replaceParameter = $this->maskService->mask($replaceParameter, $productMask);
                 }
                 else {
                    // O código da partida é armazenado neste campo.
                    $replaceParameter = $error["NRSEQCTBTEMP"];
                 }
                 // Remove o '%s' da mensagem, substituindo pela String obtida.
                 $message = substr_replace($message, $replaceParameter, strpos($message, '%s'), 2);
             }
             $error["DSERROTEMP"] = $message;
        }
        return $errorsArray;
    }

    /*
     * Função para retornar uma String com a máscara para código de produtos.
     * Essa máscara é formatada de forma a poder ser utilizada pela função Mask
     * do utilitário Masks.
     */
    private function getProductMask() {
        $productMask = PictureProduto::mascaraProduto($this->nrorg);
        // Substitui os caracteres diferentes de '.' por '#', para que a máscara
        // seja padronizada de acordo com o formato aceito pelo utilitário Masks.
        $productMask = preg_replace("/[^.]/", "#", $productMask);
        return $productMask;
    }

    /*
     * Função para retornar um array com as partidas de um lançamento que teve
     * erros de contabilização.
     * Se não houver erros no lançamento, retorna um array vazio.
     */
    private function getPartidas($cdempresa, $nrlancctb) {
        $lancctbtemp = $this->contabilizacaoFactory->getPartidasTemporarias(array(
            "NRORG" => $this->nrorg,
            "CDEMPRESA" => $cdempresa,
            "NRLANCCTBTEMP" => $nrlancctb
        ));
        return $lancctbtemp;
    }

    /*
     * Função para retornar um array com os rateios das partidas de um
     * lançamento que teve erros de contabilização.
     * Se não houver rateio das partidas, retorna um array vazio.
     */
    private function getRateios($cdempresa, $nrlancctb) {
        $ratectbtemp = $this->contabilizacaoFactory->getRateiosTemporarios(array(
            "NRORG" => $this->nrorg,
            "CDEMPRESA" => $cdempresa,
            "NRLANCCTBTEMP" => $nrlancctb
        ));
        return $ratectbtemp;
    }

    /*
     * Função para remover os dados temporários das tabelas no banco de dados.
     * Esta função deve ser executada logo após armazenar esses dados.
     */
    private function deleteDadosTemporarios($cdempresa, $nrlancctb) {
        $params = array(
            "NRORG" => $this->nrorg,
            "CDEMPRESA" => $cdempresa,
            "NRLANCCTBTEMP" => $nrlancctb
        );
        
        $this->contabilizacaoFactory->deleteErrosTemporarios($params);
        $this->contabilizacaoFactory->deletePartidasTemporarias($params);
        $this->contabilizacaoFactory->deleteRateiosTemporarios($params);
    }
    
    private function isertDadosTemporarios($cdempresa, $nrlancctb) {
        
        $this->contabilizacaoFactory->deleteErrosTemporarios($params);
        $this->contabilizacaoFactory->deletePartidasTemporarias($params);
        $this->contabilizacaoFactory->deleteRateiosTemporarios($params);
        
    }
    
    public function verificaErroContab($dataSets, Response $response) {
        try{
           
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $cdempresa = $dataSets[0]['CDEMPRESA'];
            $nrlancctbtemp = $dataSets[1]['NRLANCCTBTEMP'];
            $dataSet = $dataSets[2]['DATASET'];
            $tipobaixa = $dataSets[3]['TIPOBAIXA'];

            $this->deleteDadosTemporarios($cdempresa, $nrlancctbtemp);
            foreach($dataSet[0] as $lancamento){
                
                if($lancamento['IDDEBCRE'] === 'CR'){
                    $lancamento['VRLANCCTBTEMP'] = floatval ($lancamento['VRLANCCTBTEMP']);
                } else if($lancamento['IDDEBCRE'] === 'DB'){
                    $lancamento['VRLANCCTBTEMP'] = floatval ((-1)*$lancamento['VRLANCCTBTEMP']);
                }
                
                if($lancamento['IDLANCCTBCOME'] === 'N' ){
                    $lancamento['CDLANCCTBCOME'] = null;
                }
                else if($lancamento['IDLANCCTBCOME'] === "B"){
                    $lancamento['CDLANCCTBCOME'] = isset($lancamento['CDREDUCONT']) ? $lancamento['CDREDUCONT'] : $lancamento['CDLANCCTBCOME'];
                } 
                else if($lancamento['IDLANCCTBCOME'] === "F"){
                    $lancamento['CDLANCCTBCOME'] = isset($lancamento['CDFORNECED']) ? $lancamento['CDFORNECED'] : $lancamento['CDLANCCTBCOME'];
                }
                else if($lancamento['IDLANCCTBCOME'] === "C"){
                    $lancamento['CDLANCCTBCOME'] = isset($lancamento['CDCLIENTE']) ? $lancamento['CDCLIENTE'] : $lancamento['CDLANCCTBCOME'];
                }
                
                if(isset($lancamento['DSHISTPADR']) && $lancamento['DSHISTPADR'] != ""){
                    $lancamento['DSHISTCTB'] = $lancamento['DSHISTPADR'];
                }
                
                $this->insertLancTemp($lancamento);
            }
            //CONTADOR PARA ADICIONAR O NRSEQRATTEMP CORRETAMENTE
            $contador = 0;
            
            foreach($dataSet[1] as $rateio){
                $rateio['NRSEQRATTEMP'] = str_pad($contador, 5, "0", STR_PAD_LEFT);
                $contador++;
                
                foreach($dataSet[0] as $lancamento){
                    if($lancamento['NRSEQCTBTEMP'] == $rateio['NRSEQCTBTEMP']){
                        $rateio['IDDEBCRE'] = $lancamento['IDDEBCRE'];
                    }
                }
                
                if(isset($rateio['IDDEBCRE'])){
                    if($rateio['IDDEBCRE'] === 'CR'){
                        $rateio['VRRATECTBTEMP'] = floatval ($rateio['VRRATECTBTEMP']);
                    } else if($rateio['IDDEBCRE'] === 'DB'){
                        $rateio['VRRATECTBTEMP'] = floatval ((-1)*$rateio['VRRATECTBTEMP']);
                    }
                }
                //Adicionado indice cdmepresa e nrlancctbtemp
                if(!isset($rateio['CDEMPRESA'])){
                    $rateio['CDEMPRESA'] = $cdempresa;
                }
                if(!isset($rateio['NRLANCCTBTEMP'])){
                    $rateio['NRLANCCTBTEMP'] = $nrlancctbtemp;
                }
                
                $this->insertRateioTemp($rateio);
            }
            
            $this->contabilizacaoFactory->callProcedureVerificaErroCtb($cdempresa, $nrlancctbtemp);
            
            $this->checkErrosContabilizacao($cdempresa, $nrlancctbtemp);
            
            $connection->commit();
            
            return true;
            
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->saveContabilizacaoData($response);
            throw $e;
        }
        
    }
    
    public function insertLancTemp($row){
        $params = array(
            'CDEMPRESA'     => $row['CDEMPRESA'],
            'NRLANCCTBTEMP' => $row['NRLANCCTBTEMP'],
            'NRSEQCTBTEMP'  => $row["NRSEQCTBTEMP"],
            'DTLANCCTBTEMP' => $row['DTLANCCTBTEMP'],
            'IDLANCCTBCOME' => $row['IDLANCCTBCOME'] ? $row['IDLANCCTBCOME']: 'N',
            'CDCONTCTBL'    => isset($row['CDCONTCTBL']) ? $row['CDCONTCTBL'] : null,
            'CDHISTPADR'    => isset($row['CDHISTPADR']) ? $row['CDHISTPADR'] : null,
            'CDDOCUTEMP'    => isset($row['CDDOCUTEMP']) ? $row['CDDOCUTEMP'] : null,
            'CDLANCCTBCOME' => isset($row['CDLANCCTBCOME']) ? $row['CDLANCCTBCOME'] : null,
            'DSHISTCTB'     => isset($row['DSHISTCTB']) ? $row['DSHISTCTB'] : null,
            'VRLANCCTBTEMP' => isset($row['VRLANCCTBTEMP']) ? $row['VRLANCCTBTEMP'] : null,
            'NRORG'         => $this->nrorg,
            'IDATIVO'       => 'S'
        );
        
        $this->contabilizacaoFactory->insertLancTemp($params);
    }
    
    public function insertRateioTemp($row){

        $params = array(
            'CDCENTCUST'    => $row['CDCENTCUST'],
            'CDEMPRESA'     => $row['CDEMPRESA'],
            'CDFILIAL'      => $row['CDFILIAL'],
            'DTCOMPCTBTEMP' => $row['DTCOMPCTBTEMP'],
            'NRLANCCTBTEMP' => $row['NRLANCCTBTEMP'],
            'NRSEQCTBTEMP'  => $row['NRSEQCTBTEMP'],
            'NRSEQRATTEMP'  => $row['NRSEQRATTEMP'],
            'VRRATECTBTEMP' => $row['VRRATECTBTEMP'],
            'NRORG'         => $this->nrorg,
            'IDATIVO'       => 'S'
        );
        
        $this->contabilizacaoFactory->insertRateioTemp($params);
    }
    
    public function insertErroTemp($row){
        $params = array(
            'CDEMPRESA'     => $row['CDEMPRESA'],
            'CDERROTEMP'    => $row['CDERROTEMP'],
            'NRLANCCTBTEMP' => $row['NRLANCCTBTEMP'],
            'NRSEQCTBTEMP'  => $row['NRSEQCTBTEMP'],
            'NRORG'         => $this->nrorg,
            'IDATIVO'       => 'S'     
        );
        
        $this->contabilizacaoFactory->insertErroTemp($params);
    }
    
    public function gravaLancContab($dataSets){
        try{
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $cdempresa = $dataSets[0]['CDEMPRESA'];
            $nrlancctbtemp = $dataSets[1]['NRLANCCTBTEMP'];
            
            $this->contabilizacaoFactory->callProcedureGravaLancCtb($cdempresa, $nrlancctbtemp);
            
            $connection->commit();
            
            return true;
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
}